@@ -11,12 +11,18 @@ from .hydrogram import gen_hydrogram
 from .gtif import openf, as_array, get_rowcol
 from .geotransform import GeoTransformFit
 from .debug import print_exception, progress_counter  # , crono, truncate, progress_bar
-# from .geotransform import GeoTransformFit
 
 
 class MFD(Matrix):
 
-    def __init__(self, dtm_path: str, mannings_path: str, nodata: float = -999, mute: bool = True):
+    def __init__(
+        self,
+        dtm_path: str,
+        mannings_path: str,
+        nodata: float = -99,
+        radius: float = 2000,
+        mute: bool = True
+    ) -> None:
         self.dtm_ds = openf(dtm_path)
         self.dtm_gt = self.dtm_ds.GetGeoTransform()
         self.mannings_ds = openf(mannings_path)
@@ -35,9 +41,10 @@ class MFD(Matrix):
         self.mannings = GeoTransformFit(
             self.mannings,
             self.mannings_gt,
-            self.dtm_gt
+            self.dtm_gt,
         )
 
+        self.radius = radius
         self.mute = mute
 
     def __del__(self):
@@ -210,6 +217,7 @@ class MFD(Matrix):
             else:
                 progress = lambda i, f: f
 
+            distance = 0
             while True:
                 try:
                     flood = next(hyd)
@@ -228,14 +236,35 @@ class MFD(Matrix):
                 # steps.append(len(next_step))
                 # news.append(len(list(filter(lambda k: k not in last_step, next_step))))
                 # lens.append(self.array([math.sqrt(sum(coord**2)) for coord in abs(self.argwhere(floods > 0) - start) * self.cellsize]).max())
-                if len([rc for rc in next_step if rc not in last_step]) > 0:
-                    trapped = 0
-                else:
+                # if len([rc for rc in next_step if rc not in last_step]) > 0:
+                #     trapped = 0
+                # else:
+                #     trapped += 1
+                edge = self.array([math.sqrt(sum(coord**2)) for coord in abs(self.argwhere(floods > 0) - start) * self.cellsize]).max()
+                if distance == int(edge):
                     trapped += 1
-                # distance = self.array([math.sqrt(sum(coord**2)) for coord in abs(self.argwhere(floods > 0) - start) * self.cellsize]).max()
+                else:
+                    trapped = 0
+
+                distance = int(edge)
+
                 last_flood = flood
                 i += 1
-                if self.is_over or i > 1e+5 or len(next_step) == 0 or trapped >= 1000:
+                # if self.is_over or i > 1e+5 or len(next_step) == 0 or trapped >= 1000 or distance >= self.radius:
+                # print(f"iteration: {i}")
+                # print(f"trapped: {trapped}")
+                # print(f"distance: {distance}")
+                if self.is_over:
+                    print("\nExit condition: Flood is over dtm boundaries")
+                    break
+                elif i > 1e+4:
+                    print("\nExit condition: Max recursion limit")
+                    break
+                elif trapped >= 5e+3:
+                    print("\nExit condition: Flood's stability reached")
+                    break
+                elif distance >= self.radius:
+                    print("\nExit condition: Distance limit reached")
                     break
 
         except KeyboardInterrupt:
